import logging

# from django.contrib.auth.models import Group as GroupModel
from django.db.models.signals import m2m_changed, post_save
from django.dispatch import receiver

from .... import app_settings
from ....permissions import is_django_staff
from ..models import Group
from ...profiles.models import Profile

logger = logging.getLogger('django_sso_app.core.apps.groups.signals')


@receiver(m2m_changed)
def signal_handler_when_user_is_added_or_removed_from_group(action, instance, pk_set, model, **kwargs):

    if model == Group and instance.__class__ == Profile:
        if kwargs.get('raw', False):
            # https://github.com/django/django/commit/18a2fb19074ce6789639b62710c279a711dabf97
            return

        user = instance.user
        profile = instance

        logger.info('Groups signal')

        is_loaddata = getattr(user, '__dssoa__loaddata', False)
        is_creating = getattr(user, '__dssoa__creating', False)
        is_entering_incomplete_group = getattr(user, '__dssoa__enter_incomplete_group', False)
        is_exiting_incomplete_group = getattr(user, '__dssoa__exit_incomplete_group', False)

        must_update_rev = (not is_loaddata) and \
                          (not is_creating) and \
                          (not is_entering_incomplete_group) and \
                          (not is_exiting_incomplete_group)

        groups_updated = False

        if action == 'pre_add':
            groups_updated = True
            for pk in pk_set:
                _group = Group.objects.get(id=pk)
                logger.info('Profile "{}" entered group "{}"'.format(profile, _group))

        elif action == 'pre_remove':
            groups_updated = True
            for pk in pk_set:
                _group = Group.objects.get(id=pk)
                logger.info('Profile "{}" exited from group "{}"'.format(profile, _group))

        # updating rev

        if groups_updated and must_update_rev:
            profile.update_rev(True)


@receiver(post_save, sender=Profile)
def profile_updated(sender, instance, created, **kwargs):
    """
    Profile model has been updated,
    """
    profile = instance

    if kwargs['raw']:
        # https://github.com/django/django/commit/18a2fb19074ce6789639b62710c279a711dabf97
        return

    should_manage_groups = app_settings.BACKEND_ENABLED or app_settings.REPLICATE_PROFILE

    if should_manage_groups and not is_django_staff(profile.user):
        if created:  # if instance.pk:
            if profile.is_incomplete:
                logger.debug('Profile model has been created incomplete, entering "incomplete" group')

                setattr(profile.user, '__dssoa__enter_incomplete_group', True)

                group, _created = Group.objects.get_or_create(name='incomplete')

                profile.groups.add(group)
        else:
            if not profile.is_incomplete:
                group, _created = Group.objects.get_or_create(name='incomplete')

                if profile.groups.filter(name=group.name).count() > 0:
                    logger.debug('Profile model has been completed, exiting "incomplete" group')

                    setattr(profile.user, '__dssoa__exit_incomplete_group', True)

                    profile.groups.remove(group)

        # check profile groups
        user_object_profile = getattr(profile.user, '__dssoa__profile__object', None)

        if user_object_profile is not None:
            group_names = user_object_profile.get('groups', [])

            for group_name in group_names:
                group, _created = Group.objects.get_or_create(name=group_name)

                if profile.groups.filter(name=group.name).count() == 0:
                    profile.groups.add(group)



import logging

from django.contrib.auth.signals import user_logged_in
from django.dispatch import receiver

from ....utils import set_session_key

logger = logging.getLogger('django_sso_app.core.apps.users.signals')


@receiver(user_logged_in)
def post_user_login(sender, user, request, **kwargs):

    set_session_key(request, '__dssoa__requesting_user', user)

    logger.debug('Users, "{}" LOGGED IN!!!'.format(user))



from django.contrib.auth import get_user_model

from django_filters import rest_framework as filters

User = get_user_model()


class UserFilter(filters.FilterSet):
    class Meta:
        model = User
        fields = {
            'username': ['exact'],
            'sso_app_profile__sso_id': ['exact'],
            'groups__name': ['exact'],
        }



import logging

from django.dispatch import receiver
from django.db.models.signals import post_save, pre_save
from django.contrib.auth import get_user_model
from django.contrib.auth.signals import user_logged_in
from django.utils.timezone import now

from ..utils import get_or_create_user_profile
from ..models import Profile

logger = logging.getLogger('django_sso_app.core.apps.profiles.signals')

User = get_user_model()


@receiver(pre_save, sender=Profile)
def profile_pre_updated(sender, instance, **kwargs):
    """
    Profile model has been updated, updating rev
    """
    if kwargs['raw']:
        # https://github.com/django/django/commit/18a2fb19074ce6789639b62710c279a711dabf97
        return

    profile = instance

    # set profile completed_at if all required fields are set
    if not profile.is_incomplete and profile.completed_at is None:
        profile.completed_at = now()

    if not profile._state.adding:
        user = profile.user

        if getattr(user, '__dssoa__apigateway_update', False):
            logger.debug('Profile has been updted by api gateway, skipping rev update')
        elif getattr(user, '__dssoa__creating', False):
            logger.debug('Created, skipping rev update')
        else:
            logger.debug('Profile model has been updated, updating rev')
            profile.update_rev(False)


@receiver(post_save, sender=Profile)
def profile_updated(sender, instance, created, **kwargs):
    """
    Profile model has been updated, updating rev
    """
    profile = instance

    if kwargs['raw']:
        # https://github.com/django/django/commit/18a2fb19074ce6789639b62710c279a711dabf97
        setattr(profile.user, '__dssoa__loaddata', True)
        return

    if not created:  # if instance.pk:
        logger.debug('Profile model has been updated, refreshing instance')
        profile.refresh_from_db()

# user

@receiver(post_save, sender=User)
def create_update_user_profile(sender, instance, created, **kwargs):
    if kwargs['raw']:
        # https://github.com/django/django/commit/18a2fb19074ce6789639b62710c279a711dabf97
        # setting property flag
        setattr(instance, '__dssoa__loaddata', True)
        return

    user = instance

    if created:
        logger.debug('backend signals user created')

        setattr(instance, '__dssoa__creating', True)

        profile = get_or_create_user_profile(user, Profile, commit=True)

        logger.debug('new profile created "{}"'.format(profile))

        # refreshing user instance
        user.sso_app_profile = profile

    else:
        logger.debug('backend signals user updated')
        profile = get_or_create_user_profile(user, Profile, commit=False)
        update_rev = True

        # replaced by '__user_loggin_in'
        # if user.previous_serialized_as_string == user.serialized_as_string:
        if getattr(user, '__dssoa__creating', False):
            logger.info('user _creating')
            update_rev = False
        elif getattr(user, '__dssoa__user_loggin_in', False):
            logger.info('user loggin in')
            update_rev = False
        elif user.password_has_been_hardened:
            logger.info('password_has_been_hardened')
            update_rev = False

        if user.email_has_been_updated:
            # aligning sso_app_profile django_user_email
            profile.django_user_email = user.sso_app_new_email

        if user.username_has_been_updated:
            # aligning sso_app_profile django_user_username
            profile.django_user_username = user.sso_app_new_username

        if update_rev:
            logger.info('Update rev by User signal while user fields have been updated')
            profile.update_rev(True)  # updating rev


@receiver(user_logged_in)
def post_user_login(**kwargs):
    """
    Post login profile creation safety
    :param kwargs:
    :return:
    """
    user = kwargs['user']

    _profile = get_or_create_user_profile(user, Profile)
    # setattr(user, 'sso_app_profile', _profile)

    logger.debug('Profile, "{}" LOGGED IN!!!'.format(_profile))
